@@ -1,9 +1,14 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 """A pure Python3 implementation of the LSV2 protocol"""
-from .client import LSV2
+from .client import *
 from .const import *
+from .dat_cls import *
 from .table_reader import *
 from .translate_messages import *
+from .err import *
 
-__version__ = "0.7.7"
+__version__ = "1.0.0"
+__author__ = "drunsinn"
+__license__ = "MIT"
+__email__ = "dr.unsinn@googlemail.com"
